"""
功能: 移除文件日期, 用创建日期替代.
命令: filedate.exe [FILES ...]
特性:
 - 日期格式为: YYYYMMDD
 - 文件分隔符包含: -|_|#|.|~
"""

import argparse
import concurrent.futures
import pathlib
import re
import sys
import time
import typing

# 检测分隔符
SEPARATORS = "-_#.~"

# 处理后使用分隔符
SEP = "_"


def remove_date_prefix(filename: str) -> str:
    """移除形式如 `YYYYMMDD` 的日期格式.

    Args:
        filename (str): 原始文件名

    Returns:
        移除日期后文件名

    >>> remove_date_prefix('20211211_hello.txt')
    'hello.txt'
    >>> remove_date_prefix('20191112-my-file.xls')
    'my-file.xls'
    >>> remove_date_prefix('20201211my-file.xls')
    'my-file.xls'
    >>> remove_date_prefix('2022-my-file.xls')
    '2022-my-file.xls'
    """
    pattern = re.compile(
        r"(20|19)\d{2}((0[1-9])|(1[012]))((0[1-9])|([12]\d)|(3[01]))"
    )
    match = re.search(pattern, filename)

    if not match:
        return filename

    b, e = match.start(), match.end()
    if b >= 1 and filename[b - 1] in SEPARATORS:
        filename = filename.replace(filename[b - 1 : e], "")
    elif e + 1 <= len(filename) - 1 and filename[e] in SEPARATORS:
        filename = filename.replace(filename[b : e + 1], "")
    else:
        filename = filename.replace(filename[b:e], "")
    return remove_date_prefix(filename)


def rename_target(filepath: pathlib.Path) -> typing.Tuple[str, str]:
    """更新日期标识, 如果没有则创建, 按照 YYYYMMDD 格式

    Args:
        filepath: 文件路径

    Returns:
        修改后的路径
    """
    modified, created = filepath.stat().st_mtime, filepath.stat().st_ctime
    time_mark = time.strftime(
        "%Y%m%d", time.localtime(max((modified, created)))
    )
    dst_name = filepath.with_name(
        f"{time_mark}{SEP}{remove_date_prefix(filepath.name)}"
    )
    filepath.rename(dst_name)
    return filepath.name, dst_name.name


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("files", nargs="+", help="待处理文件列表")

    args = parser.parse_args()
    files: typing.List[str] = args.files

    if not len(files):
        print("[!] 输入参数不正确")
        parser.print_help(sys.stdout)
        return

    targets = [pathlib.Path(f) for f in files if pathlib.Path(f).exists()]
    t0 = time.perf_counter()
    rets: typing.List[concurrent.futures.Future] = []
    print("[*] 启动线程")
    with concurrent.futures.ThreadPoolExecutor() as t:
        for target in targets:
            tt = "文件" if target.is_file() else "目录"
            print(f"[*] 开始处理{tt}=[{str(target)}]...")
            rets.append(t.submit(rename_target, target))
    print("[*] 关闭线程")

    for future in concurrent.futures.as_completed(rets):
        old, new = future.result()
        print(f"[*] 重命名文件成功: {old} => {new}")
    print(
        f"[*] 目标数=[{len(targets)}]\n用时=[{time.perf_counter() - t0:.3f}]s."
    )


if __name__ == "__main__":
    main()
